@@ -5,13 +5,14 @@
 import itertools
 import math
 from collections import OrderedDict, namedtuple
+from enum import Enum
 from textwrap import dedent
 from typing import Dict, Iterable, List, Literal, Optional, Tuple, Union
 
 import numpy
 from affine import Affine
 
-from .crs import CRS, MaybeCRS, norm_crs
+from .crs import CRS, CRSMismatchError, MaybeCRS, SomeCRS, norm_crs
 from .geom import (
     BoundingBox,
     Geometry,
@@ -23,12 +24,14 @@ from .geom import (
     point,
     polygon_from_transform,
 )
-from .math import clamp, is_affine_st, is_almost_int
+from .math import clamp, is_affine_st, is_almost_int, snap_grid
+from .roi import Tiles as RoiTiles
 from .roi import align_up, polygon_path, roi_normalise, roi_shape
 from .types import (
     XY,
     Index2d,
     MaybeInt,
+    NormalizedROI,
     Resolution,
     Shape2d,
     SomeIndex2d,
@@ -45,29 +48,23 @@ OutlineMode = Union[
     Literal["native"], Literal["pixel"], Literal["geo"], Literal["auto"]
 ]
 
+
+class AnchorEnum(Enum):
+    """
+    Defines which way to snap geobox pixel grid.
+    """
+
+    EDGE = 0
+    CENTER = 1
+    FLOATING = 2
+
+
+GeoboxAnchor = Union[AnchorEnum, XY[float]]
+
 # pylint: disable=invalid-name,too-many-public-methods,too-many-lines
 Coordinate = namedtuple("Coordinate", ("values", "units", "resolution"))
 
 
-def _align_pix(left: float, right: float, res: float, off: float) -> Tuple[float, int]:
-    if res < 0:
-        res = -res
-        val = math.ceil((right - off) / res) * res + off
-        width = max(1, int(math.ceil((val - left - 0.1 * res) / res)))
-    else:
-        val = math.floor((left - off) / res) * res + off
-        width = max(1, int(math.ceil((right - val - 0.1 * res) / res)))
-    return val, width
-
-
-def _align_pix_tight(left: float, right: float, res: float) -> Tuple[float, int]:
-    if res < 0:
-        res = -res
-        return right, max(1, int(math.ceil((right - left - 0.1 * res) / res)))
-
-    return left, max(1, int(math.ceil((right - left - 0.1 * res) / res)))
-
-
 class GeoBox:
     """
     Defines the location and resolution of a rectangular grid of data,
@@ -96,39 +93,57 @@ class GeoBox:
         tight: bool = False,
         shape: Optional[SomeShape] = None,
         resolution: Optional[SomeResolution] = None,
-        align: Optional[XY[float]] = None,
+        anchor: GeoboxAnchor = AnchorEnum.EDGE,
+        tol: float = 0.01,
     ) -> "GeoBox":
         """
         Construct :py:class:`~odc.geo.geobox.GeoBox` from a bounding box.
 
         :param bbox: Bounding box in CRS units, lonlat is assumed when ``crs`` is not supplied
         :param crs: CRS of the bounding box (defaults to EPSG:4326)
-        :param tight: Supplying ``tight=True`` turns off pixel snapping.
         :param shape: Span that many pixels.
         :param resolution: Use specified resolution
-        :param align:
-        :return: :py:class:`~odc.geo.geobox.GeoBox` that covers supplied bounding box.
-        """
+        :param tight: Supplying ``tight=True`` turns off pixel snapping.
+        :param anchor:
+            By default snaps grid such that pixel edges fall on X/Y axis. Ignored when tight mode is
+            used.
+        :param tol:
+            Fraction of a pixel that can be ignored, defaults to 1/100. Bounding box of the output
+            geobox is allowed to be smaller than supplied bounding box by that amount.
 
-        if align is None and tight is False:
-            align = xy_(0, 0)
-        if crs is None:
-            crs = "EPSG:4326"
+        :return:
+           :py:class:`~odc.geo.geobox.GeoBox` that covers supplied bounding box.
+        """
+        # pylint: disable=too-many-locals
+
+        _snap: Optional[XY[float]] = None
+
+        if tight:
+            anchor = AnchorEnum.FLOATING
+
+        if isinstance(anchor, XY):
+            _snap = anchor
+        if anchor == AnchorEnum.EDGE:
+            _snap = xy_(0, 0)
+        elif anchor == AnchorEnum.CENTER:
+            _snap = xy_(0.5, 0.5)
 
         if not isinstance(bbox, BoundingBox):
-            bbox = BoundingBox(*bbox)
+            bbox = BoundingBox(*bbox, crs=(crs or "epsg:4326"))
+        elif bbox.crs is None:
+            bbox = BoundingBox(*bbox.bbox, crs=(crs or "epsg:4326"))
 
         if resolution is not None:
             rx, ry = res_(resolution).xy
-            if align is None:
-                offx, nx = _align_pix_tight(bbox.left, bbox.right, rx)
-                offy, ny = _align_pix_tight(bbox.bottom, bbox.top, ry)
+            if _snap is None:
+                offx, nx = snap_grid(bbox.left, bbox.right, rx, None, tol=tol)
+                offy, ny = snap_grid(bbox.bottom, bbox.top, ry, None, tol=tol)
             else:
-                offx, nx = _align_pix(bbox.left, bbox.right, rx, align.x)
-                offy, ny = _align_pix(bbox.bottom, bbox.top, ry, align.y)
+                offx, nx = snap_grid(bbox.left, bbox.right, rx, _snap.x, tol=tol)
+                offy, ny = snap_grid(bbox.bottom, bbox.top, ry, _snap.y, tol=tol)
 
             affine = Affine.translation(offx, offy) * Affine.scale(rx, ry)
-            return GeoBox((ny, nx), crs=crs, affine=affine)
+            return GeoBox((ny, nx), crs=bbox.crs, affine=affine)
 
         if shape is None:
             raise ValueError("Must supply shape or resolution")
@@ -138,14 +153,14 @@ class GeoBox:
         rx = bbox.span_x / nx
         ry = -bbox.span_y / ny
 
-        if align is None:
+        if _snap is None:
             offx, offy = bbox.left, bbox.top
         else:
-            offx, _ = _align_pix(bbox.left, bbox.right, rx, align.x)
-            offy, _ = _align_pix(bbox.bottom, bbox.top, ry, align.y)
+            offx, _ = snap_grid(bbox.left, bbox.right, rx, _snap.x, tol=tol)
+            offy, _ = snap_grid(bbox.bottom, bbox.top, ry, _snap.y, tol=tol)
 
         affine = Affine.translation(offx, offy) * Affine.scale(rx, ry)
-        return GeoBox((ny, nx), crs=crs, affine=affine)
+        return GeoBox((ny, nx), crs=bbox.crs, affine=affine)
 
     @staticmethod
     def from_geopolygon(
@@ -153,35 +168,42 @@ class GeoBox:
         resolution: SomeResolution,
         crs: MaybeCRS = None,
         align: Optional[XY[float]] = None,
+        *,
+        anchor: GeoboxAnchor = AnchorEnum.EDGE,
+        tol: float = 0.01,
     ) -> "GeoBox":
         """
         Construct :py:class:`~odc.geo.geobox.GeoBox` from a polygon.
 
         :param resolution:
            Either a single number or a :py:class:`~odc.geo.types.Resolution` object.
+
         :param crs:
            CRS to use, if different from the geopolygon
+
         :param align:
-           Align geobox such that point 'align' lies on the pixel boundary.
+            Deprecated: please switch to ``anchor=``
+
+        :param anchor:
+            By default snaps grid such that pixel edges fall on X/Y axis.
+
+        :param tol:
+            Fraction of a pixel that can be ignored, defaults to 1/100. Bounding box of the output
+            geobox is allowed to be smaller than supplied bounding box by that amount.
+
         """
         resolution = res_(resolution)
-        if align is None:
-            align = xy_(0.0, 0.0)
-
-        assert (
-            0.0 <= align.x <= abs(resolution.x)
-        ), "X align must be in [0, abs(x_resolution)] range"
-        assert (
-            0.0 <= align.y <= abs(resolution.y)
-        ), "Y align must be in [0, abs(y_resolution)] range"
-
+        if align is not None:
+            # support old-style "align", which is basically anchor but in CRS units
+            ax, ay = align.xy
+            anchor = xy_(ax / abs(resolution.x), ay / abs(resolution.y))
         if crs is None:
             crs = geopolygon.crs
         else:
             geopolygon = geopolygon.to_crs(crs)
 
         return GeoBox.from_bbox(
-            geopolygon.boundingbox, crs, resolution=resolution, align=align
+            geopolygon.boundingbox, crs, resolution=resolution, anchor=anchor, tol=tol
         )
 
     def buffered(self, xbuff: float, ybuff: Optional[float] = None) -> "GeoBox":
@@ -237,7 +259,7 @@ class GeoBox:
 
     def is_empty(self) -> bool:
         """Check if geobox is "empty"."""
-        return self._shape.shape == (0, 0)
+        return 0 in self._shape
 
     def __bool__(self) -> bool:
         return not self.is_empty()
@@ -245,6 +267,22 @@ class GeoBox:
     def __hash__(self):
         return hash((*self._shape, self._crs, self._affine))
 
+    def overlap_roi(self, other: "GeoBox", tol: float = 1e-8) -> NormalizedROI:
+        """
+        Compute overlap as ROI.
+
+        Figure out slice into this geobox that shares pixels with the ``other`` geobox with
+        consistent pixel grid.
+
+        :raises:
+            :py:class:`ValueError` when two geoboxes are not pixel-aligned.
+        """
+        nx, ny = self._shape.xy
+        x0, y0, x1, y1 = map(int, bounding_box_in_pixel_domain(other, self, tol))
+        x0, y0 = max(0, x0), max(0, y0)
+        x1, y1 = min(x1, nx), min(y1, ny)
+        return numpy.s_[y0:y1, x0:x1]
+
     @property
     def transform(self) -> Affine:
         """Linear mapping from pixel space to CRS."""
@@ -323,12 +361,12 @@ class GeoBox:
         xs = numpy.arange(nx) * rx + (tx + rx / 2)
         ys = numpy.arange(ny) * ry + (ty + ry / 2)
 
-        units = self._crs.units if self._crs is not None else ("1", "1")
+        crs_units = self._crs.units if self._crs is not None else ("1", "1")
 
         return OrderedDict(
             (dim, Coordinate(labels, units, res))
             for dim, labels, units, res in zip(
-                self.dimensions, (ys, xs), units, (ry, rx)
+                self.dimensions, (ys, xs), crs_units, (ry, rx)
             )
         )
 
@@ -341,20 +379,41 @@ class GeoBox:
         self._extent = _extent
         return _extent
 
+    @property
+    def boundingbox(self) -> BoundingBox:
+        """GeoBox bounding box in the native CRS."""
+        return BoundingBox.from_transform(self._shape, self._affine, crs=self._crs)
+
     def _reproject_resolution(self, npoints: int = 100):
         bbox = self.extent.boundingbox
         span = max(bbox.span_x, bbox.span_y)
         return span / npoints
 
+    def footprint(
+        self, crs: SomeCRS, buffer: float = 0, npoints: int = 100
+    ) -> Geometry:
+        """
+        Compute footprint in foreign CRS.
+
+        :param crs: CRS of the destination
+        :param buffer: amount to buffer in source pixels before transforming
+        :param npoints: number of points per-side to use, higher number
+                        is slower but more accurate
+        """
+        assert self.crs is not None
+        ext = self.extent
+        if buffer > 0:
+            buffer = buffer * max(*self.resolution.xy)
+            ext = ext.buffer(buffer)
+
+        return ext.to_crs(crs, resolution=self._reproject_resolution(npoints))
+
     @property
     def geographic_extent(self) -> Geometry:
         """GeoBox extent in EPSG:4326."""
         if self._crs is None or self._crs.geographic:
             return self.extent
-
-        return self.extent.to_crs(
-            CRS("EPSG:4326"), resolution=self._reproject_resolution()
-        )
+        return self.footprint("epsg:4326")
 
     coords = coordinates
     dims = dimensions
@@ -491,6 +550,26 @@ class GeoBox:
         """
         return self * Affine.translation(tx, ty)
 
+    @property
+    def left(self) -> "GeoBox":
+        """Same size geobox to the left of this one."""
+        return self.translate_pix(-self.shape.x, 0)
+
+    @property
+    def right(self) -> "GeoBox":
+        """Same size geobox to the right of this one."""
+        return self.translate_pix(self.shape.x, 0)
+
+    @property
+    def top(self) -> "GeoBox":
+        """Same size geobox directly above this one."""
+        return self.translate_pix(0, -self.shape.y)
+
+    @property
+    def bottom(self) -> "GeoBox":
+        """Same size geobox directly below this one."""
+        return self.translate_pix(0, self.shape.y)
+
     def rotate(self, deg: float) -> "GeoBox":
         """
         Rotate GeoBox around the center.
@@ -532,6 +611,13 @@ class GeoBox:
             raise ValueError(raise_error)
         return False
 
+    @property
+    def axis_aligned(self):
+        """
+        Check if Geobox is axis-aligned (not rotated).
+        """
+        return self._confirm_axis_aligned()
+
     @property
     def center_pixel(self) -> "GeoBox":
         """
@@ -574,13 +660,16 @@ class GeoBox:
         from .ui import pick_grid_step  # pylint: disable=import-outside-toplevel
 
         nx, ny = self._shape.xy
-        if step == 0:
-            step = pick_grid_step(max(nx, ny))
-        xx = [*range(0, nx, step), nx]
-        yy = [*range(0, ny, step), ny]
-        vertical = [list(itertools.product([x], yy)) for x in xx[1:-1]]
-        horizontal = [list(itertools.product(xx, [y])) for y in yy[1:-1]]
-        lines = multiline(vertical + horizontal, self._crs)
+        if nx > 0 and ny > 0:
+            if step == 0:
+                step = pick_grid_step(max(nx, ny))
+            xx = [*range(0, nx, step), nx]
+            yy = [*range(0, ny, step), ny]
+            vertical = [list(itertools.product([x], yy)) for x in xx[1:-1]]
+            horizontal = [list(itertools.product(xx, [y])) for y in yy[1:-1]]
+            lines = multiline(vertical + horizontal, self._crs)
+        else:
+            lines = multiline([], self._crs)
 
         if mode == "pixel":
             return lines
@@ -776,7 +865,7 @@ def bounding_box_in_pixel_domain(
 
     tx, ty = round(tx), round(ty)
     ny, nx = geobox.shape
-    return BoundingBox(tx, ty, tx + nx, ty + ny)
+    return BoundingBox(tx, ty, tx + nx, ty + ny, None)
 
 
 def geobox_union_conservative(geoboxes: List[GeoBox]) -> GeoBox:
@@ -816,11 +905,19 @@ def geobox_intersection_conservative(geoboxes: List[GeoBox]) -> GeoBox:
     # standardise empty geobox representation
     if bbox.left > bbox.right:
         bbox = BoundingBox(
-            left=bbox.left, bottom=bbox.bottom, right=bbox.left, top=bbox.top
+            left=bbox.left,
+            bottom=bbox.bottom,
+            right=bbox.left,
+            top=bbox.top,
+            crs=bbox.crs,
         )
     if bbox.bottom > bbox.top:
         bbox = BoundingBox(
-            left=bbox.left, bottom=bbox.bottom, right=bbox.right, top=bbox.bottom
+            left=bbox.left,
+            bottom=bbox.bottom,
+            right=bbox.right,
+            top=bbox.bottom,
+            crs=bbox.crs,
         )
 
     affine = reference.affine * Affine.translation(*bbox[:2])
@@ -913,13 +1010,8 @@ class GeoboxTiles:
         :param box: source :py:class:`~odc.geo.GeoBox`
         :param tile_shape: Shape of sub-tiles in pixels ``(rows, cols)``
         """
-        tile_shape = shape_(tile_shape)
         self._gbox = box
-        self._tile_shape = tile_shape
-        ny, nx = (
-            int(math.ceil(float(N) / n)) for N, n in zip(box.shape, tile_shape.shape)
-        )
-        self._shape = shape_((ny, nx))
+        self._tiles = RoiTiles(box.shape, tile_shape)
         self._cache: Dict[Index2d, GeoBox] = {}
 
     @property
@@ -928,22 +1020,21 @@ class GeoboxTiles:
         return self._gbox
 
     @property
-    def shape(self):
+    def shape(self) -> Shape2d:
         """Number of tiles along each dimension."""
-        return self._shape
+        return self._tiles.shape
 
-    def _idx_to_slice(self, idx: Index2d) -> Tuple[slice, slice]:
-        def _slice(i, N, n) -> slice:
-            _in = i * n
-            if 0 <= _in < N:
-                return slice(_in, min(_in + n, N))
-            raise IndexError(f"Index {idx} is out of range")
+    @property
+    def roi(self) -> RoiTiles:
+        """
+        Access ROI covered by tile.
 
-        ir, ic = (
-            _slice(i, N, n)
-            for i, N, n in zip(idx.yx, self._gbox.shape, self._tile_shape.yx)
-        )
-        return (ir, ic)
+        .. code-block:: python
+
+            gbt = GeoboxTiles(..)
+            roi = gbt.roi[0, 3]
+        """
+        return self._tiles
 
     def chunk_shape(self, idx: SomeIndex2d) -> Shape2d:
         """
@@ -953,18 +1044,7 @@ class GeoboxTiles:
         :returns: ``(nrows, ncols)`` shape of a tile (edge tiles might be smaller)
         :raises: :py:class:`IndexError` when index is outside of ``[(0,0) -> .shape)``.
         """
-        idx = iyx_(idx)
-
-        def _sz(i: int, n: int, tile_sz: int, total_sz: int) -> int:
-            if 0 <= i < n - 1:  # not edge tile
-                return tile_sz
-            if i == n - 1:  # edge tile
-                return total_sz - (i * tile_sz)
-            # out of index case
-            raise IndexError(f"Index {idx} is out of range")
-
-        n1, n2 = map(_sz, idx.yx, self._shape.yx, self._tile_shape.yx, self._gbox.shape)
-        return shape_((n1, n2))
+        return self._tiles.tile_shape(idx)
 
     def __getitem__(self, idx: SomeIndex2d) -> GeoBox:
         """
@@ -979,7 +1059,7 @@ class GeoboxTiles:
         if sub_gbox is not None:
             return sub_gbox
 
-        roi = self._idx_to_slice(idx)
+        roi = self._tiles[idx]
         return self._cache.setdefault(idx, self._gbox[roi])
 
     def range_from_bbox(self, bbox: BoundingBox) -> Tuple[range, range]:
@@ -994,12 +1074,15 @@ class GeoboxTiles:
             _out = clamp(math.ceil(v2), 0, N)
             return range(_in, _out)
 
-        sy, sx = self._tile_shape.yx
+        if bbox.crs != self._gbox.crs:
+            raise CRSMismatchError()
+
+        sy, sx = self._tiles.tile_shape((0, 0)).yx
         A = Affine.scale(1.0 / sx, 1.0 / sy) * (~self._gbox.transform)
         # A maps from X,Y in meters to chunk index
         bbox = bbox.transform(A)
 
-        NY, NX = self._shape
+        NY, NX = self._tiles.shape.yx
         xx = clamped_range(bbox.left, bbox.right, NX)
         yy = clamped_range(bbox.bottom, bbox.top, NY)
         return (yy, xx)
