@@ -16,17 +16,28 @@ from ._xr_interop import (
     ODCExtensionDa,
     ODCExtensionDs,
     assign_crs,
+    rasterize,
     register_geobox,
     spatial_dims,
+    wrap_xr,
     xr_coords,
+    xr_reproject,
+    xr_zeros,
 )
 
+wrap = wrap_xr
+
 __all__ = (
     "ODCExtension",
     "ODCExtensionDa",
     "ODCExtensionDs",
     "assign_crs",
+    "rasterize",
     "register_geobox",
     "spatial_dims",
     "xr_coords",
+    "wrap",
+    "wrap_xr",
+    "xr_reproject",
+    "xr_zeros",
 )
